//DMatrix.h
#ifndef DMATRIX
#define DMATRIX

//---------------------------------------------------------------------------
template <class T> class DMatrix {
        int col_count, row_count;
        T* matrix;
public:
        DMatrix();
        DMatrix(int row_size, int col_size);
        DMatrix(const DMatrix<T>& Source);
        ~DMatrix();
        bool IsValid(int row_index, int col_index);
        T& operator ()(int row_index, int col_index);
        void Resize(const int iSize, const int jSize);
        int Get_ColCount() {return col_count;}
        int Get_RowCount() {return row_count;}
};
//---------------------------------------------------------------------------
template <class T> DMatrix<T>::DMatrix() {
        col_count = row_count = 1;
        matrix = new T[col_count * row_count];
        if (!matrix)
                throw "    .";
}

template <class T> DMatrix<T>::DMatrix(int row_size, int col_size) {
        if (row_size < 1 || col_size < 1)
                throw "  ";
        col_count = col_size;
        row_count = row_size;
        matrix = new T[col_count * row_count];
        if (!matrix)
                throw "    .";
}

template <class T> DMatrix<T>::DMatrix(const DMatrix<T>& Source) {
        col_count = Source.col_count;
        row_count = Source.row_count;
        int n = col_count * row_count;
        matrix = new T[n];
        if (!matrix)
                throw "    .";
        while (n--)
                matrix[n] = Source.matrix[n];
}

template <class T> DMatrix<T>::~DMatrix() {
        if (matrix) {
                delete [] matrix;
                matrix = NULL;
                col_count = 0;
                row_count = 0;
        }
}

template <class T> bool DMatrix<T>::IsValid (int row_index, int col_index) {
        return (row_index >= 0 && row_index < row_count &&
                col_index >= 0 && col_index < col_count)?true:false;
}

template <class T> T& DMatrix<T>::operator () (int row_index, int col_index) {
        if (!IsValid(row_index, col_index))
       	        throw "    .";

        return matrix[row_index * col_count + col_index];
}

template <class T> void DMatrix<T>::Resize(const int iSize, const int jSize) {
        if (iSize <= 0 || jSize <= 0)
                throw "  ";
        if (iSize == row_count && jSize == col_count)
                return;
        if (matrix)
        {
                T* new_matrix = new T[iSize * jSize];
                if (!new_matrix)
                        throw "    -   .";

                int max_i = (iSize < row_count)?iSize:row_count;
                int max_j = (jSize < col_count)?jSize:col_count;
                for (int j = 0; j < max_j; j++)
                        for (int i = 0; i < max_i; i++)
                                new_matrix[i * jSize + j] = matrix[i * col_count + j];

                delete [] matrix;
                matrix = new_matrix;
                row_count = iSize;
                col_count = jSize;
        }
};

#endif  //DMATRIX
