//stack.h
#ifndef STACK_H
#define STACK_H

/******************************************************************************/
template <class T> class stack {
protected:
        int size;
public:
        virtual void Push(T TData) = 0;
        virtual T Pop() = 0;
        virtual T Top() = 0;
        int GetSize() { return size; }
};
/******************************************************************************/
template <class T> struct stack_element {
	T data;
	stack_element *next;
};
//   
template <class T> class stack_list: public stack<T> {
	stack_element<T> *head_of_stack;
	stack_element<T> *tail_of_stack;
public:
      	stack_list();
        ~stack_list();
        void Clear();
      	void Push(T TData);
        T Pop();
        T Top();
};

//   
template <class T> class stack_array: public stack<T> {
      	T* array;
        int part;
        int current;
public:
      	stack_array();
        stack_array(int part_size);
        ~stack_array();
      	void Push(T TData);
        T Pop();
        T Top();
        bool IsFull();
        void Resize();
        void Resize(int nSize);
};

/******************************************************************************/

template <class T> stack_list<T>::stack_list() {
	head_of_stack = tail_of_stack = NULL;
        size = 0;
}

template <class T> stack_list<T>::~stack_list() {
        Clear();
}

template <class T> void stack_list<T>::Clear() {
        stack_element<T> *p1, *p2;

        if (head_of_stack)
        {
                p2 = tail_of_stack;
                do
	        {
                        p1 = p2;
                        p2 = p2->next;
                        delete p1;
        	}
                while (p2 != head_of_stack && p2 != NULL);
                delete p2;
        }
        head_of_stack = NULL;
        tail_of_stack = NULL;
        size = 0;
}

template <class T> void stack_list<T>::Push(T TData) {
	stack_element<T> *p;

	p = new stack_element<T>;
	if (!p)
		throw "  .    .";

	p->data = TData;
	size++;
	if (head_of_stack)
                p->next = tail_of_stack;
	else {
	        head_of_stack = p;
                head_of_stack->next = NULL;
        }
	tail_of_stack = p;
}

template <class T> T stack_list<T>::Pop() {
	stack_element<T> *p;
        T ret;

	if (!head_of_stack)
	        throw " .";

        ret = tail_of_stack->data;
	p = tail_of_stack;
        tail_of_stack = tail_of_stack->next;
        if (!tail_of_stack)
                head_of_stack = NULL;
        delete p;
        size--;

        return ret;
}

template <class T> T stack_list<T>::Top() {
	if (!head_of_stack)
	        throw " .";

        return tail_of_stack->data;
}

/******************************************************************************/

template <class T> stack_array<T>::stack_array() {
        size = part = 1;
        current = -1;
        array = new T[size];
        if (!array)
                throw "    .";
}

template <class T> stack_array<T>::stack_array(int part_size) {
        size = part = part_size;
        current = -1;
        array = new T[size];
        if (!array)
                throw "    .";
}

template <class T> stack_array<T>::~stack_array() {
        if (array) {
                delete [] array;
                array = NULL;
                size = 0;
                current = -1;
        }
}

template <class T> bool stack_array<T>::IsFull() {
        return (current == size - 1)?true:false;
}

template <class T> void stack_array<T>::Push(T TData) {
        if (IsFull())
                Resize();

        current++;
        array[current] = TData;
}

template <class T> T stack_array<T>::Pop() {
        if (current < 0)
                throw " .";

        return array[current--];
}

template <class T> T stack_array<T>::Top() {
        if (current < 0)
                throw " .";

        return array[current];
}

template <class T> void stack_array<T>::Resize() {
        if (array) {
                T* new_array;

                new_array = new T[size + part];
                if (!new_array)
                        throw "    -   .";

                CopyMemory(new_array, array, sizeof(T) * (current + 1));

                delete [] array;
                array = new_array;
                size += part;
        }
}

template <class T> void stack_array<T>::Resize(int nSize) {
        if (nSize >= size)
                Resize();
        else                
        if (array) {
                T* new_array;

                new_array = new T[nSize];
                if (!new_array)
                        throw "    -   .";

                CopyMemory(new_array, array, sizeof(T) * nSize);

                delete [] array;
                array = new_array;
                size = nSize;
                current = size - 1;
        }
}

/******************************************************************************/

#endif /* STACK_H */
