//string.h
#ifndef STRING_H
#define STRING_H

/******************************************************************************/
class string {
        char* str;
        int str_length;
        int _strlen(const char* s);
public:
        string();
        string(string* s);
        string(const char* s);
        ~string();
        int Length() { return str_length; }
        bool IsEmpty() { return (str_length > 0)?false:true; }
        bool IsValid(int index) { return (index >= 0 && index < str_length)?true:false; }
        string& operator = (string& s);
        string& operator = (char* s);
        bool operator == (string& s);
        char& operator [] (int index);
        char* String() { return str;}
        string& operator + (string& s);
        int Pos(char* c);
        int SubStr(char* sub_str);
        void Insert(char* s, int from);
};
/******************************************************************************/

string::string() {
        str_length = 1;
        str = new char[1];
        if (!str)
                throw "  .   .";
        str[0] = '\0';
}

string::string(string* s) {
        str_length = s->Length();
        str = new char[str_length+1];
        if (!str)
                throw "  .   .";
        str[str_length] = '\0';
        memcpy(str, s->str, str_length);
}

string::string(const char* s) {
        str_length = _strlen(s);
        str = new char[str_length+1];
        if (!str)
                throw "  .   .";
        str[str_length] = '\0';
        memcpy(str, s, str_length);
}

int string::_strlen(const char* s) {
        int len = 0;
        while(s[len])
                len++;
        return len;
}

string::~string() {
        if (str)
                delete [] str;
}

string& string::operator = (string& s) {
        if (this == &s)
                return *this;
        delete [] str;
        str_length = s.Length();
        char* new_str = new char[str_length + 1];
        if (!new_str)
                throw "  .   .";
        new_str[str_length] = '\0';
        memcpy(new_str, s.str, s.Length()+1);
        str = new_str;
        return *this;
}

string& string::operator = (char* s) {
        str_length = _strlen(s);
        char* new_str = new char[str_length + 1];
        if (!new_str)
                throw "  .   .";
        new_str[str_length] = '\0';
        memcpy(new_str, s, str_length);
        delete [] str;
        str = new_str;
        return *this;
}

bool string::operator == (string& s) {
        if (str_length != s.Length())
                return false;
        else
                return !memcmp(str, s.str, str_length);
}

char& string::operator [] (int index) {
        if (!IsValid(index))
       	        throw "    .";

        return str[index];
};

string& string::operator + (string& s) {
        str_length += s.Length();
        char* new_str = new char[str_length + 1];
        if (!new_str)
                throw "  .   .";
        new_str[str_length] = '\0';
        memcpy(new_str, str, str_length - s.Length());
        memcpy(new_str + str_length - s.Length(), s.str, s.Length());
        delete [] str;
        str = new_str;
        return *this;
}

int string::Pos(char* c) {
        if(_strlen(c) == 0)
                return -1;

        int i;
        for(i = 0; i < str_length; i++)
                if(str[i] == *c)
                        break;
        return(i < str_length?i:-1);
}

int string::SubStr(char* sub_str) {
        int len = _strlen(sub_str);
        if(len == 0 || len > str_length)
                return -1;
        int n = str_length - 1;
        char* s = new char[n+1];
        if (!s)
                throw "  .   .";
        s[n] = '\0';
        memcpy(s, str, n+1);
        while(n-- && memcmp(s++, sub_str, len))
                if(n+1 < len)
                {
                         if(n == 0)
                                  delete [] (s - str_length + 1);
                         else
                                  delete [] (s - str_length + n + 1);
                         return -1;
                }

        delete [] (s - str_length + n + 1);
        return(n < 0?-1:str_length - n - 2);
}

void string::Insert(char* s, int from) {
        if(!IsValid(from))
                throw "  .";

        from++;

        char* new_str = new char[str_length + _strlen(s) + 1];
        if (!new_str)
                throw "  .   .";
        new_str[str_length + _strlen(s)] = '\0';
        if (from - 1 < 0)
                memcpy(new_str, s, _strlen(s));
        else
        {
                memcpy(new_str, str, from - 1);
                memcpy(new_str + from - 1, s, _strlen(s));
                memcpy(new_str + from + _strlen(s) - 1, str + from - 1, str_length - from + 1);
        }
        delete [] str;
        str = new_str;
        str_length += _strlen(s);
}

/******************************************************************************/

#endif /* STRING_H */

